@@ -29,6 +29,7 @@
 
 #include <pan/general/foreach.h>
 #include <pan/general/quark.h>
+#include <pan/general/map-vector.h>
 #include <pan/general/sorted-vector.h>
 #include <pan/usenet-utils/numbers.h>
 #include <pan/data/article.h>
@@ -42,6 +43,8 @@
 
 namespace pan
 {
+  typedef std::vector<const Article*> articles_t;
+
   /**
    * File-based implementation of the `Data' backend interface.
    *
@@ -100,7 +103,7 @@ namespace pan
          Server(): port(119), article_expiration_age(31), max_connections(2), rank(1) {}
       };
 
-      typedef std::map<Quark,Server> servers_t;
+      typedef Loki::AssocVector<Quark,Server> servers_t;
 
       servers_t _servers;
 
@@ -195,7 +198,7 @@ namespace pan
           unsigned long _xover_high;
           Server(): _xover_high(0) {}
         };
-        typedef std::map<Quark,Server> servers_t;
+        typedef Loki::AssocVector<Quark,Server> servers_t;
         servers_t _servers;
 
         unsigned long _article_count;
@@ -220,7 +223,7 @@ namespace pan
         }
       };
 
-      typedef std::map<Quark,ReadGroup> read_groups_t;
+      typedef Loki::AssocVector<Quark,ReadGroup> read_groups_t;
       read_groups_t _read_groups;
 
       ReadGroup* find_read_group (const Quark& g) {
@@ -321,6 +324,7 @@ namespace pan
       };
 
       typedef std::map<Quark,ArticleNode*> nodes_t;
+      typedef std::vector<ArticleNode*> nodes_v;
 
       struct GroupHeaders
       {
@@ -346,6 +350,7 @@ namespace pan
         const Article* find_top_existing_ancestor (const Quark& mid) const;
         Article* find_article (const Quark& mid);
         const Article* find_article (const Quark& mid) const;
+        void find_nodes (const quarks_t& mids, nodes_v& setme);
         void remove_articles (const quarks_t& mids);
         void build_references_header (const Article* article, std::string& setme) const;
       };
@@ -364,7 +369,7 @@ namespace pan
       static ArticleNode* find_ancestor (ArticleNode * node,
                                          const Quark & ancestor_mid);
 
-      typedef std::map<Quark,GroupHeaders*> group_to_headers_t;
+      typedef Loki::AssocVector<Quark,GroupHeaders*> group_to_headers_t;
       group_to_headers_t _group_to_headers;
 
       GroupHeaders* get_group_headers (const Quark& group);
@@ -400,12 +405,16 @@ namespace pan
           void add_articles (const quarks_t&, bool apply_filter=true);
           void remove_articles (const quarks_t& mids);
 
+        private:
+          void accumulate_descendants (quarks_t&, const ArticleNode*) const;
+
         private: // implementation fields
           const Quark _group;
           DataImpl & _data;
           nodes_t _nodes;
           std::deque<ArticleNode> _node_chunk;
           FilterInfo _filter;
+          struct NodeMidCompare;
       };
 
       std::set<MyTree*> _trees;
@@ -500,7 +509,7 @@ namespace pan
         XOverEntry(): _batch_parts_size(0), refcount(0) { }
       };
 
-      typedef std::map<Quark,XOverEntry> xovers_t;
+      typedef Loki::AssocVector<Quark,XOverEntry> xovers_t;
       xovers_t _xovers;
 
       XOverEntry * _cached_xover_entry;
@@ -524,16 +533,16 @@ namespace pan
 
       virtual void xover_ref     (const Quark          & group);
 
-      virtual bool xover_add     (const Quark          & server,
-                                  const Quark          & group,
-                                  const StringView     & subject,
-                                  const StringView     & author,
-                                  const time_t           date,
-                                  const Quark          & message_id,
-                                  const StringView     & references,
-                                  const unsigned long    byte_count,
-                                  const unsigned long    line_count,
-                                  const StringView     & xref);
+      virtual const Article* xover_add  (const Quark          & server,
+                                         const Quark          & group,
+                                         const StringView     & subject,
+                                         const StringView     & author,
+                                         const StringView     & date,
+                                         const StringView     & message_id,
+                                         const StringView     & references,
+                                         const unsigned long    byte_count,
+                                         const unsigned long    line_count,
+                                         const StringView     & xref);
 
       virtual void xover_unref   (const Quark          & group);
 
