@@ -119,6 +119,27 @@ DataImpl :: GroupHeaders :: find_article (const Quark& message_id)
   return a;
 }
 
+void
+DataImpl :: GroupHeaders :: find_nodes (const quarks_t& mids, nodes_v& setme)
+{
+  setme.reserve (mids.size());
+  nodes_t::iterator node_it  (_nodes.begin()), node_end (_nodes.end());
+  quarks_t::const_iterator mid_it (mids.begin()), mid_end (mids.end());
+
+  while (node_it!=node_end && mid_it!=mid_end)
+  {
+    if (node_it->first < *mid_it)
+      ++node_it;
+    else if (*mid_it < node_it->first) // looks like we won't find *mid_it
+      ++mid_it;
+    else { // equal
+      setme.push_back (node_it->second);
+      ++node_it;
+      ++mid_it;
+    }
+  }
+}
+
 void
 DataImpl :: GroupHeaders :: remove_articles (const quarks_t& mids)
 {
