@@ -19,6 +19,7 @@
 
 #include <config.h>
 #include <cassert>
+#include <glib/gmessages.h> // for g_assert
 #include <pan/general/debug.h>
 #include <pan/general/foreach.h>
 #include <pan/general/quark.h>
@@ -93,7 +94,7 @@ DataImpl :: MyTree :: set_filter (const Data::ShowType    show_type,
   // apply the filter to everything in the group...
   articles_t all, pass, fail;
   const GroupHeaders * h =  _data.get_group_headers (_group);
-  assert (h != 0);
+  g_assert (h != 0);
   foreach_const (nodes_t, h->_nodes, it)
     if (it->second->_article)
       all.push_back (it->second->_article);
@@ -267,105 +268,184 @@ namespace
   }
 }
 
+struct
+DataImpl :: MyTree :: NodeMidCompare
+{
+  bool operator ()(const ArticleNode* a, const ArticleNode* b) const {
+    return a->_mid < b->_mid;
+  }
+  bool operator ()(const Quark& a, const std::pair<const pan::Quark, pan::DataImpl::ArticleNode*>& b) const {
+    return a < b.first;
+  }
+  bool operator ()(const std::pair<const pan::Quark, pan::DataImpl::ArticleNode*>& a, const Quark& b) const {
+    return a.first < b;
+  }
+  bool operator ()(const std::pair<const pan::Quark, pan::DataImpl::ArticleNode*>& a,
+                   const std::pair<const pan::Quark, pan::DataImpl::ArticleNode*>& b) const {
+    return a.first < b.first;
+  }
+};
+
+void
+DataImpl :: MyTree :: accumulate_descendants (quarks_t& descendants, const ArticleNode* node) const
+{
+  descendants.insert (node->_mid);
+  foreach_const (ArticleNode::children_t, node->_children, it)
+    accumulate_descendants (descendants, *it);
+}
+
 void
 DataImpl :: MyTree :: add_articles (const quarks_t& mids, bool need_to_filter)
 {
-  //std::cerr << LINE_ID << " adding " << mids.size()<< " articles to " << this << "..." << std::endl;
+  NodeMidCompare compare;
 
-  // make a list of the mids already in the tree...
-  typedef std::vector<Quark> quark_v;
-  quark_v old_mids (_nodes.size());
-  int i = 0;
-  foreach_const (nodes_t, _nodes, it)
-    old_mids[i++] = it->first; // these quarks are already sorted by virtue of being keys in a map...
-  //std::cerr << LINE_ID << " previously there were " << old_mids.size() << " articles..." << std::endl;
-
-  // we don't want to add articles that we already have...
-  quark_v new_mids;
-  new_mids.reserve (mids.size());
+  // prune out any articles already in the tree
+  //std::cerr << LINE_ID << " mids.size() " << mids.size() << std::endl;
+  quarks_t new_mids;
   std::set_difference (mids.begin(), mids.end(),
-                       old_mids.begin(), old_mids.end(),
-                       inserter (new_mids, new_mids.begin()));
-  //std::cerr << LINE_ID << ' ' << new_mids.size() << " of the new articles aren't already in the tree..." << std::endl;
+                       _nodes.begin(), _nodes.end(),
+                       inserter (new_mids, new_mids.begin()),
+                       compare);
+  //std::cerr << LINE_ID << " after removing duplicates: " << new_mids.size() << std::endl;
 
-  // only add articles which pass the filter
+  // convert it to articles
   GroupHeaders * h (_data.get_group_headers (_group));
-  assert (h);
+  g_assert (h);
+  nodes_v nodes;
+  h->find_nodes (new_mids, nodes);
+  
+  // prune out any articles that don't pass the filter
   if (need_to_filter) {
-    quark_v tmp;
-    tmp.reserve (new_mids.size());
-    foreach_const (quark_v, new_mids, it) {
-      Article * a (h->find_article (*it));
-      if (_data._article_filter.test_article (_data, _filter, _group, *a))
+    nodes_v tmp;
+    tmp.reserve (nodes.size());
+    foreach_const (nodes_v, nodes, it)
+      if (_data._article_filter.test_article (_data, _filter, _group, *((*it)->_article)))
         tmp.push_back (*it);
-    }
-    new_mids.swap (tmp); // now new_mids only holds those passing the filter
-    //std::cerr << LINE_ID << ' ' << new_mids.size() << " survived the remove-duplicates and filter tests." << std::endl;
+    nodes.swap (tmp);
   }
-    
+  //std::cerr << LINE_ID << " after filtering: " << nodes.size() << std::endl;
+      
   // construct new Nodes for the new mids...
   int node_index (_node_chunk.size());
   _node_chunk.resize (_node_chunk.size() + new_mids.size());
-  foreach_const (quark_v, new_mids, it) {
-    Article * a (h->find_article (*it));
+  nodes_v my_nodes;
+  my_nodes.reserve (nodes.size());
+  foreach_const (nodes_v, nodes, it) {
+    Article * a ((*it)->_article);
     ArticleNode * node (&_node_chunk[node_index++]);
     node->_mid = a->message_id;
-    node->_article = a;
+    node->_article = const_cast<Article*>(a);
+//std::cerr << LINE_ID << " added " << node->_mid << " to the tree (unthreaded so far)" << std::endl;
     std::pair<nodes_t::iterator,bool> result (_nodes.insert (std::pair<Quark,ArticleNode*>(node->_mid, node)));
-    assert (result.second); // freshly added; not a duplicate
+    g_assert (result.second); // freshly added; not a duplicate
+    my_nodes.push_back (node);
   }
 
-  // reparent everybody...
+  // find parents for the new articles
+
   ArticleTree::Diffs diffs;
-  unique_sorted_quarks_t all_mids;
-  all_mids.reserve (old_mids.size() + new_mids.size());
-  set_union (old_mids.begin(), old_mids.end(),
-             new_mids.begin(), new_mids.end(),
-             std::back_inserter (all_mids.get_container()));
-  foreach_const (unique_sorted_quarks_t, all_mids, it)
+  for (size_t i(0), n(nodes.size()); i!=n; ++i)
   {
-    ArticleNode * hchild (h->_nodes.find (*it)->second);
-    ArticleNode * hparent (find_closest_ancestor (hchild, all_mids));
-    ArticleNode * tchild (_nodes.find (*it)->second);
-    ArticleNode * tparent (hparent ? _nodes.find (hparent->_mid)->second : 0);
+    const ArticleNode * node (nodes[i]);
+    ArticleNode * tree_node (my_nodes[i]);
+    g_assert (node != tree_node);
+    g_assert (node->_mid == tree_node->_mid);
+    g_assert (node->_article == tree_node->_article);
 
-    if (tchild->_parent != tparent) // if the parent changed:
-    {
-      ArticleTree::Diffs::Reparent r;
-      r.message_id = *it;
+    Diffs::Added added;
+    added.message_id = tree_node->_mid;
 
-      if (tchild->_parent) { // unparent from the old
-        r.old_parent = tchild->_parent->_mid;
-        tchild->_parent->_children.remove (tchild);
-        tchild->_parent = 0;
-      }
+    // of its ancestors,
+    // find the first one present in our tree
+    ArticleNode * parent (0);
+    const nodes_t::const_iterator nend (_nodes.end());
+    for (const ArticleNode *it(node->_parent); it && !parent; it=it->_parent) {
+      nodes_t::iterator nit (_nodes.find (it->_mid));
+      if (nit != nend)
+        parent = nit->second;
+    }
 
-      if (tparent) { // reparent to the new
-        r.new_parent = tparent->_mid;
-        tchild->_parent = tparent;
-        tparent->_children.push_front (tchild);
-      }
+    if (parent) {
+      tree_node->_parent = parent;
+      parent->_children.push_back (tree_node);
+      added.parent = parent->_mid;
+    }
 
-      if (!contains (old_mids, r.message_id)) // new article...
-      {
-        debug ("I think that " << r.message_id << " is a new article");
-        ArticleTree::Diffs::Added a;
-        a.message_id = r.message_id;
-        a.parent = r.new_parent;
-        diffs.added.push_back (a);
-      }
-      else // old article -- reparenting
-      {
-        debug ("I think " << r.message_id << " needs reparenting");
-        assert (r.old_parent != r.new_parent);
-        diffs.reparented.push_back (r);
+    //std::cerr << LINE_ID << " child " << added.message_id << " has parent " << added.parent << std::endl;
+    diffs.added.push_back (added);
+  }
+
+  // maybe find new parents for descendants of the new articles...
+
+  // get a list of all articles that are descendants of the new articles
+  quarks_t descendants;
+  foreach (nodes_v, nodes, it)
+    accumulate_descendants (descendants, *it);
+
+  // don't need to reparent the new nodes because we've already
+  // parented them.
+  if (1) {
+    quarks_t tmp;
+    std::set_difference (descendants.begin(), descendants.end(),
+                         new_mids.begin(), new_mids.end(),
+                         inserter (tmp, tmp.begin()));
+    descendants.swap (tmp);
+  }
+    
+  // convert that into a list of tree nodes
+  // that need to be tested for reparenting
+  nodes_v my_descendants;
+  nodes_t::iterator nit(_nodes.begin()), nend(_nodes.end());
+  quarks_t::iterator dit(descendants.begin()), dend(descendants.end());
+  while (nit!=nend && dit!=dend) {
+    if (nit->second->_mid < *dit)
+      ++nit;
+    else if (*dit < nit->second->_mid)
+      ++dit;
+    else {
+      //std::cerr << LINE_ID << " descendant " << nit->first << " may need reparenting" << std::endl;
+      my_descendants.push_back (nit->second);
+      ++nit;
+      ++dit;
+    }
+  }
+  //std::cerr << LINE_ID << " " << my_descendants.size() << " articles may need to be reparented..." << std::endl;
+
+  // reparent each of those nodes...
+  foreach (nodes_v, my_descendants, it)
+  {
+    ArticleNode * tree_node (*it);
+    ArticleNode * new_parent (0);
+
+//std::cerr << LINE_ID << " looking for a new parent for " << tree_node->_mid << std::endl;
+    
+    const ArticleNode * node (h->find_node (tree_node->_mid));
+    node = node->_parent;
+    while (node && !new_parent) {
+      //std::cerr << LINE_ID << " maybe " << node->_mid <<" can be its new parent..." << std::endl;
+      nodes_t::iterator nit (_nodes.find (node->_mid));
+      if (nit != _nodes.end())
+        new_parent = nit->second;
+      else {
+        node = node->_parent;
+        //std::cerr << LINE_ID << " but no, that parent isn't in the tree." << std::endl;
       }
     }
-    else if (contains (new_mids, *it)) // this is a new root article
-    {
-      ArticleTree::Diffs::Added a;
-      a.message_id = *it;
-      diffs.added.push_back (a);
+//std::cerr << LINE_ID << " " << tree_node->_mid << "'s best parent is " << new_parent << std::endl;
+
+    if (new_parent != tree_node->_parent) {
+      ArticleNode * old_parent (tree_node->_parent);
+      ArticleTree::Diffs::Reparent reparent;
+      if (old_parent) {
+        reparent.old_parent = old_parent->_mid;
+        old_parent->_children.remove (tree_node);
+      }
+      new_parent->_children.push_back (tree_node);
+      tree_node->_parent = new_parent;
+      reparent.message_id = tree_node->_mid;
+      reparent.new_parent = new_parent->_mid;
+//std::cerr << LINE_ID << " REPARENTED: " << reparent.message_id << " has a new parent " << reparent.new_parent << std::endl;
+      diffs.reparented.push_back (reparent);
     }
   }
 
