@@ -20,6 +20,7 @@
 #include <config.h>
 #include <cmath>
 #include <fstream>
+#include <gmime/gmime.h>
 #include <pan/general/debug.h>
 #include <pan/general/foreach.h>
 #include <pan/general/log.h>
@@ -209,27 +210,24 @@ DataImpl :: set_xover_low (const Quark   & group,
     rgs->_read.mark_range (0, low, true);
 }
 
-bool
+const Article*
 DataImpl :: xover_add (const Quark         & server,
                        const Quark         & group,
                        const StringView    & subject,
                        const StringView    & author,
-                       const time_t          time_posted,
-                       const Quark         & message_id,
+                       const StringView    & time_posted,
+                       const StringView    & message_id,
                        const StringView    & references_in,
                        const unsigned long   byte_count,
                        const unsigned long   line_count,
                        const StringView    & xref)
 {
-  bool new_article_created (false);
+  const Article* new_article (0);
   GroupHeaders * h (get_group_headers (group));
   h->_dirty = true;
   XOverEntry& workarea (xover_get_workarea (group));
-const bool d (subject == "filmy - jakie polecacie");
-if (d) std::cerr << LINE_ID << " references_in[" << references_in << ']' << std::endl;
   const std::string references (
     GNKSA :: remove_broken_message_ids_from_references (references_in));
-if (d) std::cerr << LINE_ID << " references [" << references << ']' << std::endl;
 
   /***
   **** Multipart Handling
@@ -238,10 +236,12 @@ if (d) std::cerr << LINE_ID << " references [" << references << ']' << std::endl
   int part_index, part_count;
   find_parts (subject, group, line_count, part_index, part_count);
   Quark art_mid;
-  const std::string norm (normalize_subject_for_multiparts (subject));
+  std::string norm;
 
   if (part_count > 1)
   {
+    mime :: remove_multipart_part_from_subject (subject, norm);
+
     StringView ref(references), parent;
     ref.pop_last_token (parent, ' ');
     parent.trim ();
@@ -253,7 +253,7 @@ if (d) std::cerr << LINE_ID << " references [" << references << ']' << std::endl
       const Article* candidate (h->find_article (candidate_mid));
       if ((candidate->author == author)
           && ((int)candidate->parts.size() == part_count)
-          && (h->find_parent_message_id(message_id) == parent))
+          && (h->find_parent_message_id(candidate->message_id) == parent))
         art_mid = candidate_mid;
     }
   }
@@ -263,26 +263,25 @@ if (d) std::cerr << LINE_ID << " references [" << references << ']' << std::endl
     art_mid = message_id;
 
     if (part_count > 1)
-      workarea._subject_lookup.insert(std::pair<Quark,Quark>(norm, message_id));
+      workarea._subject_lookup.insert(std::pair<Quark,Quark>(norm, art_mid));
 
     // if we don't already have this article...
     if (!h->find_article (art_mid))
     {
       //std::cerr << LINE_ID << " We didn't have this article yet, so creating an instance..." << std::endl;
-      new_article_created = true;
-
       Article& a (h->alloc_new_article());
       a.author = author;
       a.subject = subject;
-      a.message_id = message_id;
+      a.message_id = art_mid;
       a.is_binary = part_count >= 1;
       a.set_part_count (a.is_binary ? part_count : 1);
-      a.time_posted = time_posted;
+      a.time_posted = time_posted.empty() ? 0 : g_mime_utils_header_decode_date (time_posted.str, NULL);
       a.xref.insert (server, xref);
       load_article (group, &a, references);
       a.score = _article_filter.score_article (*this, workarea.score_sections, group, a); // score _after_ threading
+      new_article = &a;
 
-      workarea._added_batch.insert (message_id);
+      workarea._added_batch.insert (art_mid);
     }
   }
 
@@ -293,10 +292,8 @@ if (d) std::cerr << LINE_ID << " references [" << references << ']' << std::endl
   Article::Part part;
   part.bytes = byte_count;
   part.lines = line_count;
-  //part.time_posted = time_posted;
   part.index = part_count<2 ? 1 : part_index;
-  part.message_id = message_id;
-  //part.xref.insert (server, xref);
+  part.message_id.assign (message_id.str, message_id.len);
   load_part (group, art_mid, part);
 
   ++workarea._batch_parts_size;
@@ -308,16 +305,24 @@ if (d) std::cerr << LINE_ID << " references [" << references << ']' << std::endl
   ***  Maybe flush the batched changes
   **/
 
-  if (workarea._batch_parts_size >= 10000)
+  if (workarea._batch_parts_size >= 5000)
   //if (workarea._batch_parts_size >= 100) // torture test
   {
-//std::cerr << LINE_ID << " flushing changed size: " << workarea._changed_batch.size() << ", added size " << workarea._added_batch.size() << std::endl;
     on_articles_added (group, workarea._added_batch);
     workarea._added_batch.clear();
     on_articles_changed (workarea._changed_batch);
     workarea._changed_batch.clear();
     workarea._batch_parts_size = 0;
+
+    XOverEntry::subject_to_mid_t& s2m (workarea._subject_lookup);
+    for (XOverEntry::subject_to_mid_t::iterator it(s2m.begin()), end(s2m.end()); it!=end; ) {
+      Article * article (h->find_article (it->second));
+      if (!article || (article->get_part_state() != Article::INCOMPLETE))
+        workarea._subject_lookup.erase (it++);
+      else
+        ++it;
+    }
   }
 
-  return new_article_created;
+  return new_article;
 }
