@@ -519,19 +519,24 @@ namespace pan
       /**
        * A new header to add to a group.
        * This must be called inside an xover_ref() / xover_unref() block.
-       * Returns true if this resulted in a new article being created
-       * (as opposed to a new part being added to an existing article).
+       *
+       * If a new article was created -- as opposed to a new part
+       * being added to an existing article -- then a pointer to the
+       * new article is returned.
+       *
+       * FIXME: this return value is kind of odd, and is just there
+       * to suit task-xover's needs.
        */
-      virtual bool xover_add  (const Quark          & server,
-                               const Quark          & group,
-                               const StringView     & subject,
-                               const StringView     & author,
-                               const time_t           date,
-                               const Quark          & message_id,
-                               const StringView     & references,
-                               const unsigned long    byte_count,
-                               const unsigned long    line_count,
-                               const StringView     & xref) = 0;
+      virtual const Article* xover_add  (const Quark          & server,
+                                         const Quark          & group,
+                                         const StringView     & subject,
+                                         const StringView     & author,
+                                         const StringView     & time,
+                                         const StringView     & message_id,
+                                         const StringView     & references,
+                                         const unsigned long    byte_count,
+                                         const unsigned long    line_count,
+                                         const StringView     & xref) = 0;
 
       /**
        * The last call to xover_unref() for a group can indicate to Data
