@@ -267,7 +267,7 @@ namespace
     { "go-menu", NULL, N_("_Go") },
     { "actions-menu", NULL, N_("_Actions") },
     { "article-actions-menu", NULL, N_("_Articles") },
-    { "group-actions-menu", NULL, N_("_Groups") },
+    { "group-actions-menu", NULL, N_("G_roups") },
     { "posting-actions-menu", NULL, N_("_Post") },
     { "post-menu", NULL, N_("_Post") },
     { "help-menu", NULL, N_("_Help") },
@@ -293,7 +293,7 @@ namespace
     { "unselect-all-articles", NULL, N_("_Deselect All Articles"), "<shift><control>D", NULL, G_CALLBACK(do_unselect_all_articles) },
     { "add-subthreads-to-selection", NULL, N_("Add Su_bthreads to Selection"), "D", NULL, G_CALLBACK(do_add_subthreads_to_selection) },
     { "add-threads-to-selection", NULL, N_("Add _Threads to Selection"), "<shift>D", NULL, G_CALLBACK(do_add_threads_to_selection) },
-    { "add-similar-articles-to-selection", NULL, N_("Add _Similar Artilces to Selection"), "<control>S", NULL, G_CALLBACK(do_add_similar_to_selection) },
+    { "add-similar-articles-to-selection", NULL, N_("Add _Similar Articles to Selection"), "<control>S", NULL, G_CALLBACK(do_add_similar_to_selection) },
     { "select-article-body", NULL, N_("Select Article _Body"), NULL, NULL, G_CALLBACK(do_select_article_body) },
     { "show-preferences-dialog", GTK_STOCK_PREFERENCES, N_("Edit _Preferences..."), NULL, NULL, G_CALLBACK(do_show_preferences_dialog) },
     { "show-group-preferences-dialog", GTK_STOCK_PREFERENCES, N_("Edit _Group Preferences..."), NULL, NULL, G_CALLBACK(do_show_group_preferences_dialog) },
@@ -336,7 +336,7 @@ namespace
 
     { "post", "ICON_COMPOSE_POST", N_("_Post to Newsgroup"), "P", N_("Post to Newsgroup"), G_CALLBACK(do_post) },
     { "followup-to", "ICON_COMPOSE_FOLLOWUP", N_("_Followup to  Newsgroup"), "F", N_("Followup to Newsgroup"), G_CALLBACK(do_followup_to) },
-    { "reply-to", "ICON_COMPOSE_FOLLOWUP", N_("_Reply to Author"), "R", N_("Reply to Author"), G_CALLBACK(do_reply_to) },
+    { "reply-to", "ICON_COMPOSE_FOLLOWUP", N_("_Reply to Author in Mail"), "R", N_("Reply to Author in Mail"), G_CALLBACK(do_reply_to) },
 
     { "pan-web-page", NULL, N_("_Pan Home Page..."), NULL, NULL, G_CALLBACK(do_pan_web) },
     { "bug-report", NULL, N_("Give _Feedback or Report a Bug..."), NULL, NULL, G_CALLBACK(do_bug_report) },
