@@ -59,7 +59,14 @@ namespace pan
       bool read_more () { return read_more_or_less(true); }
       bool read_less () { return read_more_or_less(false); }
       void rot13_selected_text ();
-      GMimeMessage* create_followup ();
+      GMimeMessage* create_followup_or_reply (bool is_reply);
+
+    public:
+      GMimeMessage* get_message () {
+        if (_message)
+          g_object_ref (_message);
+        return _message;
+      }
 
     public:
       void set_character_encoding (const char * character_encoding);
@@ -76,6 +83,11 @@ namespace pan
       static void expander_activated_cb (GtkExpander*, gpointer self);
       static void foreach_part_cb (GMimeObject*, gpointer self);
       static void text_size_allocated (GtkWidget*, GtkAllocation*, gpointer);
+      static gboolean text_size_allocated_idle_cb (gpointer p);
+      void text_size_allocated_idle ();
+      void refresh_scroll_visible_state ();
+      static gboolean show_idle_cb (gpointer p);
+      static void show_cb (GtkWidget*, gpointer);
 
     private:
       GtkWidget * _expander;
@@ -86,6 +98,9 @@ namespace pan
       GtkTextBuffer * _buffer;
       GtkWidget * _root;
       GtkWidget * _text;
+      GtkWidget * _scroll;
+      bool _hscroll_visible;
+      bool _vscroll_visible;
       Article _article;
       GMimeMessage * _message;
       TextMassager _tm;
