@@ -554,7 +554,7 @@ void GUI :: do_import_tasks ()
   }
 
   if (!tasks.empty())
-    _queue.add_tasks (tasks);
+    _queue.add_tasks (tasks, Queue::BOTTOM);
 }
 
 namespace
@@ -711,7 +711,8 @@ void GUI :: do_read_selected_article ()
   if (article)
   {
     latest_read_article = *article;
-    _queue.add_task (new TaskArticle (_data, _data, *article, _cache, _data, this));
+    Task * t = new TaskArticle (_data, _data, *article, _cache, _data, this);
+    _queue.add_task (t, Queue::TOP);
 
     // maybe update the tab
     if (is_action_active ("tabbed-layout"))
@@ -932,19 +933,22 @@ void GUI :: do_supersede_article ()
   g_object_unref (content_object);
   g_object_unref (stream);
 
-  PostUI * post = new PostUI (_data, _queue, _data, _data, new_message, _prefs, _group_prefs);
-  gtk_widget_show (post->root());
+  PostUI * post = PostUI :: create_window (get_window(_root), _data, _queue, _data, _data, new_message, _prefs, _group_prefs);
+  if (post)
+  {
+    gtk_widget_show_all (post->root());
 
-  GtkWidget * w = gtk_message_dialog_new (
-    GTK_WINDOW(post->root()),
-    GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
-    GTK_MESSAGE_INFO,
-    GTK_BUTTONS_CLOSE, NULL);
-  HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(w),
-    _("Revise and send this article to replace the old one."),
-    _("Be patient!  It will take time for your changes to take effect."));
-  g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
-  gtk_widget_show (w);
+    GtkWidget * w = gtk_message_dialog_new (
+      GTK_WINDOW(post->root()),
+      GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
+      GTK_MESSAGE_INFO,
+      GTK_BUTTONS_CLOSE, NULL);
+    HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(w),
+      _("Revise and send this article to replace the old one."),
+      _("Be patient!  It will take time for your changes to take effect."));
+    g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
+    gtk_widget_show (w);
+  }
 
   g_object_unref (message);
   g_free (old_mid);
@@ -992,19 +996,22 @@ void GUI :: do_cancel_article ()
   g_object_unref (stream);
   g_free (cancel_message);
 
-  PostUI * post = new PostUI (_data, _queue, _data, _data, cancel, _prefs, _group_prefs);
-  gtk_widget_show (post->root());
+  PostUI * post = PostUI :: create_window (get_window(_root), _data, _queue, _data, _data, cancel, _prefs, _group_prefs);
+  if (post)
+  {
+    gtk_widget_show_all (post->root());
 
-  GtkWidget * w = gtk_message_dialog_new (
-    GTK_WINDOW(post->root()),
-    GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
-    GTK_MESSAGE_INFO,
-    GTK_BUTTONS_CLOSE, NULL);
-  HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(w),
-    _("Send this article to ask your server to cancel your other one."),
-    _("Be patient!  It will take time for your changes to take effect."));
-  g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
-  gtk_widget_show (w);
+    GtkWidget * w = gtk_message_dialog_new (
+      GTK_WINDOW(post->root()),
+      GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
+      GTK_MESSAGE_INFO,
+      GTK_BUTTONS_CLOSE, NULL);
+    HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(w),
+      _("Send this article to ask your server to cancel your other one."),
+      _("Be patient!  It will take time for your changes to take effect."));
+    g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
+    gtk_widget_show (w);
+  }
 
   g_object_unref (message);
 }
@@ -1047,8 +1054,9 @@ GUI :: do_post ()
   g_mime_message_set_mime_part (message, GMIME_OBJECT(part));
   g_object_unref (part);
 
-  PostUI * post = new PostUI (_data, _queue, _data, _data, message, _prefs, _group_prefs);
-  gtk_widget_show (post->root());
+  PostUI * post = PostUI :: create_window (get_window(_root), _data, _queue, _data, _data, message, _prefs, _group_prefs);
+  if (post)
+    gtk_widget_show_all (post->root());
   g_object_unref (message);
 }
 
@@ -1056,8 +1064,9 @@ void GUI :: do_followup_to ()
 {
   GMimeMessage * message = _body_pane->create_followup_or_reply (false);
   if (message) {
-    PostUI * post = new PostUI (_data, _queue, _data, _data, message, _prefs, _group_prefs);
-    gtk_widget_show (post->root());
+    PostUI * post = PostUI :: create_window(get_window(_root), _data, _queue, _data, _data, message, _prefs, _group_prefs);
+    if (post)
+      gtk_widget_show_all (post->root());
     g_object_unref (message);
   }
 }
@@ -1065,8 +1074,9 @@ void GUI :: do_reply_to ()
 {
   GMimeMessage * message = _body_pane->create_followup_or_reply (true);
   if (message) {
-    PostUI * post = new PostUI (_data, _queue, _data, _data, message, _prefs, _group_prefs);
-    gtk_widget_show (post->root());
+    PostUI * post = PostUI :: create_window (get_window(_root), _data, _queue, _data, _data, message, _prefs, _group_prefs);
+    if (post)
+      gtk_widget_show_all (post->root());
     g_object_unref (message);
   }
 }
@@ -1355,7 +1365,7 @@ void GUI :: do_xover_subscribed_groups ()
   typedef std::vector<Quark> quarks_v;
   quarks_v groups;
   _data.get_subscribed_groups (groups);
-  foreach_const (quarks_v, groups, it)
+  foreach_const_r (quarks_v, groups, it)
     _queue.add_task (new TaskXOver (_data, *it, TaskXOver::NEW));
 }
 
@@ -1370,7 +1380,7 @@ void GUI :: do_refresh_groups ()
   std::vector<Task*> tasks;
 
   const quarks_t servers (_data.get_servers ());
-  foreach_const (quarks_t, servers, it)
+  foreach_const_r (quarks_t, servers, it)
     tasks.push_back (new TaskGroups (_data, *it));
 
   if (!tasks.empty())
