@@ -419,7 +419,7 @@ HeaderPane :: render_subject (GtkTreeViewColumn * col,
 
     if (!expanded) {
       underlined = row->is_read;
-      std::snprintf (buf, sizeof(buf), "%s (%lu)", text, counter.unread_children);
+      snprintf (buf, sizeof(buf), "%s (%lu)", text, counter.unread_children);
       text = buf;
     }
   }
@@ -461,7 +461,7 @@ HeaderPane :: create_row (const EvolutionDateMaker & e,
 
 void
 HeaderPane ::  add_children_to_model (PanTreeStore               * store,
-                                      GtkTreeIter                * parent_iter,
+                                      PanTreeStore::Row          * parent_row,
                                       const Quark                & parent_mid,
                                       const Data::ArticleTree    * atree,
                                       const EvolutionDateMaker   & date_maker,
@@ -474,19 +474,15 @@ HeaderPane ::  add_children_to_model (PanTreeStore               * store,
     return;
 
   // add all these children...
-  GtkTreeIter iter;
-  std::vector<PanTreeStore::Row*> rows;
-  for (size_t i(0), count(children.size()); i!=count; ++i)
-    rows.push_back (create_row (date_maker, children[i]));
-  pan_tree_store_append_n (store, &iter, do_thread ? parent_iter : NULL, rows);
+  PanTreeStore::rows_t rows;
+  rows.reserve (children.size());
+  foreach_const (article_v, children, it)
+    rows.push_back (create_row (date_maker, *it));
+  store->append (parent_row, rows);
 
   // recurse
-  GtkTreeModel * model (GTK_TREE_MODEL(store));
-  foreach_const (article_v, children, it) {
-    const Article * article (*it);
-    add_children_to_model (store, &iter, article->message_id, atree, date_maker, do_thread);
-    gtk_tree_model_iter_next (model, &iter);
-  }
+  for (size_t i=0, n=children.size(); i<n; ++i)
+    add_children_to_model (store, rows[i], children[i]->message_id, atree, date_maker, do_thread);
 }
 
 int
@@ -667,6 +663,8 @@ HeaderPane :: set_group (const Quark& new_group)
       _atree->add_listener (this);
 
       rebuild ();
+
+      gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(_tree_view), 0, 0);
     }
   }
 
@@ -717,6 +715,7 @@ namespace
   typedef std::map<Quark,quarks_t> parent_to_children_t;
 }
 
+#if 0
 struct HeaderPane::RowInserter: public PanTreeStore::WalkFunctor
 {
   HeaderPane * header_pane;
@@ -739,8 +738,7 @@ struct HeaderPane::RowInserter: public PanTreeStore::WalkFunctor
 	const Article * article (data_tree->get_article (*it));
 	rows.push_back (header_pane->create_row (date_maker, article));
       }
-      GtkTreeIter unused;
-      pan_tree_store_append_n (store, &unused, parent, rows);
+      store->append (store->get_row(parent), rows);
     }
   }
     
@@ -752,6 +750,7 @@ struct HeaderPane::RowInserter: public PanTreeStore::WalkFunctor
     return true; // keep marching
   }
 };
+#endif
 
 namespace
 {
@@ -811,63 +810,61 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
       << diffs.reparented.size() << " reparented; "
       << diffs.removed.size() << " removed");
 
-  const bool do_sort (!diffs.added.empty()
-                      || !diffs.reparented.empty()
-                      || !diffs.removed.empty());
-  if (do_sort)
-    _tree_store->pause_sorting ();
-
-  // figure out which articles to select when this is all done.
-  // if none of the current selections survive,
-  // be ready to select the next article...
-  articles_t selectme, tmp (get_full_selection ());
-  foreach_const (articles_t, tmp, it)
-    if (!diffs.removed.count ((*it)->message_id))
-      selectme.insert (*it);
-  if (selectme.empty()) {
-    ArticleIsNotInSet tester (diffs.removed);
-    RememberArticles actor (selectme);
-    action_next_if (tester, actor);
+  const bool rows_added_or_removed (!diffs.added.empty()
+                                 || !diffs.reparented.empty()
+                                 || !diffs.removed.empty());
+  articles_t selectme;
+  if (rows_added_or_removed)
+  {
+    // figure out which articles to select when this is all done.
+    // if none of the current selections survive,
+    // be ready to select the next article...
+    articles_t tmp (get_full_selection ());
+    foreach_const (articles_t, tmp, it)
+      if (!diffs.removed.count ((*it)->message_id))
+        selectme.insert (*it);
+    if (selectme.empty()) {
+      ArticleIsNotInSet tester (diffs.removed);
+      RememberArticles actor (selectme);
+      action_next_if (tester, actor);
+    }
   }
 
   // added...
   const bool do_thread (_prefs.get_flag ("thread-headers", true));
-  const Quark empty_quark;
   if (!diffs.added.empty()) {
-    parent_to_children_t kids;
+    const EvolutionDateMaker date_maker;
+    PanTreeStore::parent_to_children_t tmp;
+    foreach_const (Data::ArticleTree::Diffs::added_t, diffs.added, it)
+      create_row (date_maker, _atree->get_article(it->message_id));
     foreach_const (Data::ArticleTree::Diffs::added_t, diffs.added, it) {
-      const Quark& parent (do_thread ? it->parent : empty_quark);
-      kids[parent].insert (it->message_id);
+      Row * parent (do_thread ? get_row (it->parent) : 0);
+      Row * child (get_row (it->message_id));
+      tmp[parent].push_back (child);
     }
-    RowInserter walk (this, _data, _atree, kids);
-    walk.append_children (_tree_store, NULL, empty_quark); // root nodes...
-    _tree_store->walk (walk);
+    _tree_store->insert_sorted (tmp);
+  }
+     
+  // reparent...
+  if (do_thread && !diffs.reparented.empty()) {
+    PanTreeStore::parent_to_children_t tmp;
+    foreach_const (Data::ArticleTree::Diffs::reparented_t, diffs.reparented, it) {
+      Row * parent (get_row (it->new_parent));
+      Row * child  (get_row (it->message_id));
+      tmp[parent].push_back (child);
+    }
+    _tree_store->reparent (tmp);
   }
 
-  // reparent...
-  if (do_thread && !diffs.reparented.empty())
-    foreach_const (Data::ArticleTree::Diffs::reparented_t, diffs.reparented, it)
-      _tree_store->reparent (get_row (it->new_parent), get_row(it->message_id));
-
   // removed...
-  if (!diffs.removed.empty())
-  {
-    std::set<Row*> rows;
+  if (!diffs.removed.empty()) {
+    PanTreeStore::rows_t rows;
+    rows.reserve (diffs.removed.size());
     foreach_const (quarks_t, diffs.removed, qit) {
-      Row * row (get_row (*qit));
-      if (row)
-        rows.insert (row);
-    }
-    std::vector<GtkTreeIter> iters;
-    iters.reserve (rows.size());
-    foreach_const (std::set<Row*>, rows, it) {
-      GtkTreeIter iter;
-      _tree_store->get_iter (*it, &iter);
-      iters.push_back (iter);
-    }
-    _tree_store->remove (&iters.front(), iters.size());
-    foreach_const (quarks_t, diffs.removed, qit)
+      rows.push_back (get_row (*qit));
       _mid_to_row.erase (*qit);
+    }
+    _tree_store->remove (rows);
   }
 
   // changed...
@@ -877,9 +874,6 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
   if (!diffs.added.empty() && _prefs.get_flag ("expand-threads-when-entering-group", false))
     gtk_tree_view_expand_all (GTK_TREE_VIEW(_tree_view));
 
-  if (do_sort)
-    _tree_store->resume_sorting ();
-
   // select the next article if one was deleted...
   // when an xover task adds a batch of articles periodically,
   // scrolling causes an irritating 'jump', but when you delete
@@ -1695,13 +1689,20 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
   _root (0),
   _tree_view (0),
   _tree_store (0),
-  _show_type (Data::SHOW_ARTICLES),
   _cache (cache)
 {
   // init the icons
   for (guint i=0; i<ICON_QTY; ++i)
     _icons[i].pixbuf = gdk_pixbuf_new_from_inline (-1, _icons[i].pixbuf_txt, FALSE, 0);
 
+  // initialize the show type...
+  const std::string show_type_str (prefs.get_string ("header-pane-show-matching", "articles"));
+  if (show_type_str == "threads")
+    _show_type = Data::SHOW_THREADS;
+  else if (show_type_str == "subthreads")
+    _show_type = Data::SHOW_SUBTHREADS;
+  else
+    _show_type = Data::SHOW_ARTICLES;
 
   // build the view...
   GtkWidget * w = _tree_view = gtk_tree_view_new ();
@@ -2212,10 +2213,8 @@ HeaderPane :: rebuild_article_action (const Quark& message_id)
 void
 HeaderPane :: on_queue_tasks_added (Queue& queue, int index, int count)
 {
-  Queue::task_states_t task_states;
-  queue.get_all_task_states (task_states);
-  for (int i(index), end(index+count); i!=end; ++i) {
-    const TaskArticle * task (dynamic_cast<const TaskArticle*>(task_states.tasks[i]));
+  for (size_t i(index), end(index+count); i!=end; ++i) {
+    const TaskArticle * task (dynamic_cast<const TaskArticle*>(queue[i]));
     if (task)
       rebuild_article_action (task->get_article().message_id);
   }
