@@ -235,7 +235,7 @@ main (int argc, char *argv[])
       std::vector<Task*> tasks;
       foreach_const (strings_t, nzb_files, it)
         NZB :: tasks_from_nzb_file (*it, nzb_output_path, cache, data, data, data, tasks);
-      queue.add_tasks (tasks);
+      queue.add_tasks (tasks, Queue::BOTTOM);
 
       // don't open the full-blown Pan, just act as a nzb client,
       // with a gui or without.
