@@ -17,9 +17,7 @@ const char * fallback_ui_file =
 "      <menuitem action='add-threads-to-selection' />\n"
 "      <menuitem action='add-similar-articles-to-selection' />\n"
 "      <menuitem action='unselect-all-articles' />\n"
-"      <separator />\n"
 "      <menuitem action='select-article-body' />\n"
-"      <menuitem action='rot13-selected-text' />\n"
 "      <separator />\n"
 "      <menuitem action='show-servers-dialog' />\n"
 "      <menuitem action='show-profiles-dialog' />\n"
@@ -64,6 +62,7 @@ const char * fallback_ui_file =
 "        <menuitem action='wrap-article-body' />\n"
 "        <menuitem action='mute-quoted-text' />\n"
 "        <menuitem action='show-all-headers' />\n"
+"        <menuitem action='rot13-selected-text' />\n"
 "        <menuitem action='monospace-font-enabled' />\n"
 "        <menuitem action='show-smilies-as-graphics' />\n"
 "        <menuitem action='show-text-markup'/>\n"
@@ -92,10 +91,29 @@ const char * fallback_ui_file =
 "      <menuitem action='read-next-thread' />\n"
 "      <menuitem action='read-previous-thread' />\n"
 "    </menu>\n"
-"    <menu action='actions-menu'>\n"
-"      <menuitem action='save-attachments' />\n"
-"      <separator />\n"
+//"    <menu action='actions-menu'>\n"
+//"      <menuitem action='save-articles' />\n"
+//"      <separator />\n"
+"      <menu action='group-actions-menu'>\n"
+"        <menuitem action='read-selected-group' />\n"
+"        <menuitem action='mark-groups-read' />\n"
+"        <separator />\n"
+"        <menuitem action='download-headers' />\n"
+"        <menuitem action='get-new-headers-in-selected-groups' />\n"
+"        <menuitem action='get-new-headers-in-subscribed-groups'/>\n"
+"        <separator />\n"
+"        <menuitem action='show-group-preferences-dialog' />\n"
+"        <separator />\n"
+"        <menuitem action='subscribe' />\n"
+"        <menuitem action='unsubscribe' />\n"
+"        <separator />\n"
+"        <menuitem action='refresh-group-list' />\n"
+"        <separator />\n"
+"        <menuitem action='delete-groups-articles' />\n"
+"      </menu>\n"
 "      <menu action='article-actions-menu'>\n"
+"        <menuitem action='save-articles' />\n"
+"        <separator />\n"
 "        <menuitem action='read-selected-article' />\n"
 "        <menuitem action='download-selected-article' />\n"
 "        <separator />\n"
@@ -110,25 +128,15 @@ const char * fallback_ui_file =
 "        <separator />\n"
 "        <menuitem action='delete-article' />\n"
 "      </menu>\n"
-"      <menu action='group-actions-menu'>\n"
-"        <menuitem action='read-selected-group' />\n"
-"        <menuitem action='download-headers' />\n"
-"        <menuitem action='mark-groups-read' />\n"
-"        <menuitem action='delete-groups-articles' />\n"
-"        <separator />\n"
-"        <menuitem action='subscribe' />\n"
-"        <menuitem action='unsubscribe' />\n"
-"        <separator />\n"
-"        <menuitem action='refresh-group-list' />\n"
-"      </menu>\n"
 "      <menu action='posting-actions-menu'>\n"
 "        <menuitem action='post' />\n"
-"        <menuitem action='followup' />\n"
+"        <menuitem action='followup-to' />\n"
+"        <menuitem action='reply-to' />\n"
 "        <separator />\n"
 "        <menuitem action='supersede-article' />\n"
 "        <menuitem action='cancel-article' />\n"
 "      </menu>\n"
-"    </menu>\n"
+//"    </menu>\n"
 "    <menu action='help-menu'>\n"
 "      <menuitem action='pan-web-page' />\n"
 "      <menuitem action='tip-jar' />\n"
@@ -147,13 +155,14 @@ const char * fallback_ui_file =
 "    <separator name='group-pane-filter'/>\n"
 "    <separator />\n"
 "    <toolitem action='post'/>\n"
-"    <toolitem action='followup'/>\n"
+"    <toolitem action='followup-to'/>\n"
+"    <toolitem action='reply-to'/>\n"
 "    <separator />\n"
 "    <toolitem action='read-more' />\n"
 "    <toolitem action='read-next-unread-article' />\n"
 "    <toolitem action='read-next-unread-thread' />\n"
 "    <separator />\n"
-"    <toolitem action='save-attachments' />\n"
+"    <toolitem action='save-articles' />\n"
 "    <separator />\n"
 "    <placeholder name='header-pane-filter' />\n"
 "    <separator />\n"
@@ -165,23 +174,26 @@ const char * fallback_ui_file =
 "  </toolbar>\n"
 "\n"
 "  <popup name='header-pane-popup'>\n"
+"    <menuitem action='save-articles' />\n"
+"    <separator />\n"
 "    <menuitem action='read-selected-article' />\n"
-"    <menuitem action='save-attachments' />\n"
+"    <menuitem action='download-selected-article' />\n"
+"    <separator />\n"
+"    <menuitem action='mark-article-read' />\n"
+"    <menuitem action='mark-article-unread' />\n"
 "    <separator />\n"
 "    <menuitem action='view-article-score' />\n"
 "    <menuitem action='add-article-score' />\n"
-"    <menuitem action='plonk' />\n"
 "    <menuitem action='watch-thread' />\n"
 "    <menuitem action='ignore-thread' />\n"
+"    <menuitem action='plonk' />\n"
 "    <separator />\n"
-"    <menuitem action='mark-article-read' />\n"
 "    <menuitem action='delete-article' />\n"
 "  </popup>\n"
 "\n"
 "  <popup name='group-pane-popup'>\n"
 "    <menuitem action='read-selected-group' />\n"
 "    <menuitem action='mark-groups-read' />\n"
-"    <menuitem action='delete-groups-articles' />\n"
 "    <separator />\n"
 "    <menuitem action='download-headers' />\n"
 "    <menuitem action='get-new-headers-in-selected-groups' />\n"
@@ -191,6 +203,10 @@ const char * fallback_ui_file =
 "    <separator />\n"
 "    <menuitem action='subscribe' />\n"
 "    <menuitem action='unsubscribe' />\n"
+"    <separator />\n"
+"    <menuitem action='refresh-group-list' />\n"
+"    <separator />\n"
+"    <menuitem action='delete-groups-articles' />\n"
 "  </popup>\n"
 "\n"
 "</ui>\n";
