@@ -158,7 +158,7 @@ namespace
     GtkListStore * store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
     const char* strings[5][2] = { { N_("Use GNOME Preferences"), "gnome" },
                                   { N_("Use KDE Preferences"), "kde" },
-                                  { N_("Use OS/X Preferences"), "mac" },
+                                  { N_("Use OS X Preferences"), "mac" },
                                   { N_("Use Windows Preferences"), "windows" },
                                   { N_("Custom Command:"), "custom" } };
     int sel_index (0);
