@@ -227,9 +227,10 @@ TaskPane :: update_status (const task_states_t& tasks)
 
   unsigned long bytes (0);
   unsigned long KiB (0);
-  foreach_const (task_states_t::states_t, tasks.states, it)
+  foreach_const (tasks_t, tasks.tasks, it)
   {
-    const Queue::TaskState state (it->second);
+    Task * task (*it);
+    const Queue::TaskState state (tasks.get_state (task));
     if (state == Queue::RUNNING)
       ++running_count;
     else if (state == Queue::STOPPED)
@@ -238,7 +239,7 @@ TaskPane :: update_status (const task_states_t& tasks)
       ++queued_count;
 
     if (state==Queue::RUNNING || state==Queue::QUEUED) {
-      bytes += it->first->get_bytes_remaining ();
+      bytes += task->get_bytes_remaining ();
       KiB += bytes / 1024ul;
       bytes %= 1024ul;
     }
