@@ -27,17 +27,22 @@
 using namespace pan;
 
 NNTP_Pool :: NNTP_Pool (const Quark        & server,
-                        const ServerInfo   & server_info,
+                        ServerInfo         & server_info,
                         Socket::Creator    * creator):
   _server_info (server_info),
   _server (server),
   _socket_creator (creator),
-  _connection_pending (false)
+  _connection_pending (false),
+  _max_connections (_server_info.get_server_limits (_server)),
+  _active_count (0)
 {
+  _server_info.add_listener (this);
 }
 
 NNTP_Pool :: ~NNTP_Pool ()
 {
+  _server_info.remove_listener (this);
+
   foreach (pool_items_t, _pool_items, it) {
     delete it->nntp->_socket;
     delete it->nntp;
@@ -65,6 +70,7 @@ NNTP_Pool :: check_out ()
     if (it->is_checked_in) {
       nntp = it->nntp;
       it->is_checked_in = false;
+      ++_active_count;
       debug ("nntp " << nntp << " is now checked out");
       break;
     }
@@ -87,6 +93,7 @@ NNTP_Pool :: check_in (NNTP * nntp, bool is_ok)
   // process the nntp if we have a match
   if (it != _pool_items.end()) {
     if (is_ok) {
+      --_active_count;
       it->is_checked_in = true;
       it->last_active_time = time (NULL);
       fire_pool_has_nntp_available ();
@@ -101,6 +108,13 @@ NNTP_Pool :: check_in (NNTP * nntp, bool is_ok)
 /***
 ****
 ***/
+void
+NNTP_Pool :: on_server_limits_changed (const Quark& server, int max_connections)
+{
+  assert (server == _server);
+  _max_connections = max_connections;
+std::cerr << LINE_ID << " pool for " << server << " knows that it has a max of " << max_connections << " connections" << std::endl;
+}
 
 void
 NNTP_Pool :: on_nntp_done (NNTP* nntp, Health health)
@@ -118,7 +132,7 @@ NNTP_Pool :: on_nntp_done (NNTP* nntp, Health health)
    {
      const std::string addr (_server_info.get_server_address (_server));
      char buf[512];
-     std::snprintf (buf, sizeof(buf), _("Unable to connect to \"%s\""), addr.c_str());
+     snprintf (buf, sizeof(buf), _("Unable to connect to \"%s\""), addr.c_str());
      fire_pool_error (buf);
    }
 
@@ -145,18 +159,9 @@ NNTP_Pool :: get_counts (int& setme_active,
                          int& setme_pending,
                          int& setme_max) const
 {
-  int idle (0);
-  int active (0);
-
-  foreach_const (pool_items_t, _pool_items, it)
-    if (it->is_checked_in)
-      ++idle;
-    else
-      ++active;
-
-  setme_active = active;
-  setme_idle = idle;
-  setme_max = _server_info.get_server_limits (_server);
+  setme_active = _active_count;
+  setme_idle = _pool_items.size() - _active_count;
+  setme_max = _max_connections;
   setme_pending  = _connection_pending ? 1 : 0;
 }
 
@@ -244,6 +249,7 @@ NNTP_Pool :: idle_upkeep ()
     // the last thing we do with the 'item' pointer.
     const time_t idle_time_secs = now - item->last_active_time;
     item->is_checked_in = false;
+    ++_active_count;
     if (idle_time_secs >= HUP_IDLE_SECS)
       item->nntp->goodbye (new NoopListener (this, true));
     else
