@@ -456,53 +456,3 @@ NNTP :: post (const StringView  & msg,
     write_next_command ();
   }
 }
-
-#if 0
-void
-nntp_cancel  (GIOChannel         * channel,
-              const PString      * message_id,
-              nntp_done_func       done_func,
-              gpointer             done_func_user_data)
-{
-	const char * newsgroups;
-	const char * author;
-	GMimeMessage * message;
-	debug_enter ("nntp_cancel");
-
-	/* sanity checks */
-	pan_return_if_fail (channel != NULL);
-	pan_return_if_fail (pstring_is_set (message_id));
-	pan_return_if_fail (acache_has_message(ACACHE_DEFAULT_KEY, message_id));
-
-	/* get info */
-	message = acache_get_message (ACACHE_DEFAULT_KEY, &message_id, 1);
-	newsgroups = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
-	author = g_mime_message_get_sender (message);
-	if (is_nonempty_string(newsgroups) && is_nonempty_string(author))
-	{
-		char * buf = g_strdup_printf (
-			"From: %s\r\n"
-			"Newsgroups: %s\r\n"
-			"Subject: cancel %s\r\n"
-			"Control: cancel %s\r\n"
-			"\r\n"
-			"Ignore\r\n"
-			"Article canceled by Pan %s\r\n",
-			author,
-			newsgroups,
-			message_id->str,
-			message_id->str,
-			VERSION);
-
-		/* post the cancel message */
-		nntp_post (channel, buf, done_func, done_func_user_data);
-
-		/* cleanup */
-		g_free (buf);
-	}
-
-	/* cleanup */
-	g_object_unref (message);
-	debug_exit ("nntp_cancel");
-}
-#endif
